-- Delay line

library ieee;
use ieee.std_logic_1164.all;		-- std_logic_vector
use ieee.std_logic_arith.all;		-- conv_std_logic
use ieee.std_logic_unsigned.all;	-- ditto
library std;
use std.textio.all;			-- to read/write files

entity delay_line is
	generic(	NA : integer := 2;	-- address width
			ND : integer := 8;	-- data width
			
			VERBOSE : integer := 0
		);

	port(	delay_line_in : in std_logic_vector( (ND-1) downto 0);
		delay_line_address : in std_logic_vector( (NA-1) downto 0);
		delay_line_sample_shift, reset, clk : in bit;
		delay_line_out : out std_logic_vector( (ND-1) downto 0)
	);
end entity;

architecture beh of delay_line is

	type memory is array (integer range <>) of std_logic_vector( (ND-1) downto 0);
	signal mem : memory( 0 to (2**NA-1));
	signal address : std_logic_vector( (NA-1) downto 0);
	signal buf : std_logic_vector( (ND-1) downto 0);
begin
--	delay_line_out <= mem( conv_integer( address));
	p : process( clk, reset, delay_line_sample_shift, delay_line_address)
		variable s : line;
	begin
--		if (reset'event and reset = '1') then
		if (reset = '1') then
			delay_line_out <= conv_std_logic_vector( 0, ND);
			address <= conv_std_logic_vector( 0, NA);
			for i in 0 to (2**NA-1) loop
				mem( i) <= (others => '0');
			end loop;
		else
--			if (delay_line_address'event) then
			if (clk'event and clk = '0') then
				buf <= mem( conv_integer( address));
			end if;
			if (clk'event and clk = '1') then
				address <= delay_line_address;
--	delay_line_out <= buf;
	delay_line_out <= mem( conv_integer( delay_line_address));
--	delay_line_out <= mem( conv_integer( address));
--					if (VERBOSE = 1) then
--						write( s, "[delay] at ");
--						write( s, now);
--						write( s, ", addr=");
--						write( s, conv_integer( address));
--						write( s, ", buf=");
--						write( s, conv_integer( buf));
--						writeline( output, s);
--					end if;
			end if;

			
--				if (delay_line_sample_shift'event and delay_line_sample_shift = '1') then
				if (delay_line_sample_shift'event and delay_line_sample_shift = '0') then
					for i in (2**NA-1) downto 1 loop
						mem( i) <= mem( i-1);
					end loop;
					mem( 0) <= delay_line_in;
--					mem( 0) <= buf;
--					if (VERBOSE = 1) then
--						write( s, "[delay] at ");
--						write( s, now);
--						write( s, ", new val=");
--						write( s, conv_integer( buf));
--						writeline( output, s);
--					end if;
				end if;

		end if;
	end process;
end architecture;

-- bizzarre

--architecture beh of delay_line is
--
--	type memory is array (integer range <>) of std_logic_vector( (ND-1) downto 0);
--	signal mem : memory( 0 to (2**NA-1));
--	signal address : std_logic_vector( (NA-1) downto 0) := conv_std_logic_vector( 0, NA);
--	signal buf : integer := 0;
--begin
--	mem0 <= mem(0);
--	delay_line_out <= mem( conv_integer( address));
--	p : process( clk, reset, delay_line_sample_shift)
--		variable s : line;
--	begin
--		if (reset'event and reset = '1') then
--			address <= conv_std_logic_vector( 0, NA);
--			for i in 0 to (2**NA-1) loop
--				mem( i) <= (others => '0');
--			end loop;
--		else
--			if (clk'event) then
--				address <= delay_line_address;
----			end if;
--			if (clk'event and clk = '0') then
----			if (clk'event) then
--				write( s, "[delay] at ");
--				write( s, now);
--				write( s, ", mem0=");
--				write( s, conv_integer( mem(0)));
--				write( s, ", mem1=");
--				write( s, conv_integer( mem(1)));
--				write( s, ", mem2=");
--				write( s, conv_integer( mem(2)));
--				write( s, ", mem3=");
--				write( s, conv_integer( mem(3)));
--				write( s, ", addr=");
--				write( s, conv_integer( address));
--				write( s, ", d_addr=");
--				write( s, conv_integer( delay_line_address));
--				writeline( output, s);
--				if (delay_line_sample_shift = '1') then
--					for i in (2**NA-1) downto 1 loop
--						mem( i) <= mem( i-1);
--					end loop;
--					mem( 0) <= delay_line_in;
--					if (VERBOSE = 1) then
--						write( s, "[delay] at ");
--						write( s, now);
--						write( s, ", new val=");
--						write( s, conv_integer( delay_line_in));
--						write( s, ", addr=");
--						write( s, conv_integer( address));
--						writeline( output, s);
--					end if;
--				end if;
----				delay_line_out <= mem( conv_integer( delay_line_address));
--			end if;
--		end if;
--	end process;
--end architecture;



