entity jiffies is
end entity;

-- behaviour with fsm-------------------------------------------------------------------------

architecture beh of jiffies is
	constant CLKTIME : time := 100 ns;
	
	signal clk : bit := '0';
	signal reset : bit := '0';
	
	signal convstb : bit := '1';
	signal dur_convstb : integer := 0;
	signal sched_convstb : integer := -1;
	
	signal jiffies : integer := 0;
	signal previous : integer;

	type myfsm is (s0, s1, s2, s3);
	signal state : myfsm := s0;
begin
	reset <= '1' after 0 ns, '0' after CLKTIME;
	
	fsm : process( clk, jiffies, reset)
	begin
		if (reset = '1') then
			state <= s0;
		elsif (clk'event) then
			case state is
				when s0 =>
					state <= s1;
				
				when s1 =>
					sched_convstb <= jiffies;
					dur_convstb <= 1;

					previous <= jiffies;
					state <= s2;
					
				when s2 =>
					if (jiffies >= previous + 14) then
						state <= s3;
					end if;
				
				when s3 =>
					state <= s1;
			end case;
		end if;
	end process;

	jiffies_p : process( clk, reset, sched_convstb, dur_convstb)
	begin
		if (reset = '1') then
			jiffies <= 0;
		else
			if ((jiffies = (sched_convstb + dur_convstb)) or (jiffies = sched_convstb)) then
				convstb <= not convstb;
			end if;
			if (clk'event) then
				jiffies <= jiffies + 1;
			end if;
		end if;
	end process;

	c : process
	begin
		clk <= not clk;
		wait for CLKTIME;
	end process;
end;

-- behaviour without fsm-------------------------------------------------------------------------

--architecture beh of jiffies is
--	constant CLKTIME : time := 100 ns;
--	
--	signal clk : bit := '0';
--	signal reset : bit := '0';
--	
--	signal convstb : bit := '1';
--	signal dur_convstb : integer := 0;
--	signal sched_convstb : integer := 0;
--	
--	signal jiffies : integer := 0;
--
--begin
--	reset <= '1' after 0 ns, '0' after CLKTIME;
--	
--	sched_convstb <= 13 after 0*CLKTIME;
--	dur_convstb <= 2;
--	
--	c : process
--	begin
--		clk <= not clk;
--		wait for CLKTIME;
--	end process;
--	
--	jiffies_p : process( clk, reset)
--	begin
--		if (reset = '1') then
--			jiffies <= 0;
--		elsif (clk'event) then
--			if ((jiffies = (sched_convstb + dur_convstb)) or (jiffies = sched_convstb)) then
--				convstb <= not convstb;
--			end if;
--			jiffies <= jiffies + 1;
--		end if;
--	end process;
--end;

