library ieee;
use ieee.std_logic_1164.all;		-- std_logic_vector
use ieee.std_logic_arith.all;		-- conv_std_logic
use ieee.std_logic_unsigned.all;	-- ditto

entity rom is
	generic(	NA : integer := 5;	-- address width
			ND : integer := 8;	-- data width
			ROM_FILE : string := "rom.txt");	-- file containing rom coefs

	port(	rom_address : in std_logic_vector( (NA-1) downto 0);
		reset, clk : in bit;
		rom_out : out std_logic_vector( (ND-1) downto 0)
		);
end entity;

--architecture beh of rom is
--	type rom_table is array (0 to 31) of STD_LOGIC_VECTOR(7 downto 0); 
--  	constant rom : rom_table :=
--	( ("00100000"), ("00011111"), ("00011110"), ("00011101"), ("00011100"), ("00011011"), ("00011010"), ("00011001"),
--	  ("00011000"), ("00010111"), ("00010110"), ("00010101"), ("00010100"), ("00010011"), ("00010010"), ("00010001"),
--	  ("00010000"), ("00001111"), ("00001110"), ("00001101"), ("00001100"), ("00001011"), ("00001010"), ("00001001"),
--	  ("00001000"), ("00000111"), ("00000110"), ("00000101"), ("00000100"), ("00000011"), ("00000010"), ("00000001")
--	);
--begin
--		
--	main : process( clk, reset, rom_address)
--	begin
--		if (reset = '1') then
--			rom_out <= (others => 'Z');
--		elsif (clk'event and clk = '1') then
--			rom_out <= rom( conv_integer( rom_address ));	
--		end if;
--	end process;
--end architecture;

------ Version qui extrait les coefficients depuis un fichier

library std;
use std.textio.all;			-- to read/write files


architecture beh0 of rom is
	constant VERBOSE : integer := 1; 		-- if 1 then print debug msgs

	type memory is array (integer range <>) of std_logic_vector( (ND-1) downto 0);
	signal rom : memory( 0 to (2**NA-1));
begin
	
	main : process( clk, reset, rom_address)
		variable s : line;
		variable num, gain : integer;
		variable name : string(10 downto 1);
		file rom_file : text open read_mode is ROM_FILE;
	begin
		if (reset = '1' and reset'event) then
			rom_out <= (others => 'Z');
			readline( rom_file, s);
			read( s, name);
			readline( rom_file, s);
			read( s, gain);
			readline( rom_file, s);
			read( s, num);
			readline( rom_file, s);
			if (VERBOSE = 1) then
				write( s, "Found ");
				write( s, name);
				write( s, " gain=");
				write( s, gain);
				write( s, " num=");
				write( s, num);
				writeline( output, s);
			end if;
			for i in 0 to (num-1) loop
				readline( rom_file, s);
				read( s, num);
				rom( i) <= conv_std_logic_vector( num, ND);
				if (VERBOSE = 1) then
					write( s, "[rom] i=");
					write( s, i);
					write( s, " n=");
					write( s, num);
--					write( s, conv_integer( rom(i)));
					writeline( output, s);
				end if;
			end loop;
		elsif (clk'event and clk = '1') then
--		elsif (clk'event) then
			rom_out <= rom( conv_integer( rom_address));
		end if;
	end process;
end architecture;





