-- test bench

library ieee;
use ieee.std_logic_1164.all;		-- std_logic_vector
use ieee.std_logic_arith.all;		-- conv_std_logic
use ieee.std_logic_unsigned.all;	-- ditto

entity tb_adc is
end entity;

architecture beh of tb_adc is
	constant CLKTIME : time := 25 ns;

	constant CONV_PULSE : time := 20 ns;
	constant ND : integer := 8;	-- data width
	constant USE_FILE : integer := 1;	-- 1 to us a sample file ; 0 to sample input
	constant ADC_FILE : string := "adc.txt";	-- file containing adc samples
	constant NUMSAMPLES : integer := 10;	-- number of samples in file

	type adc_fsm is ( init, hold, track, tracked);

component adc is
	generic(	ND : integer := ND;	-- data width
			USE_FILE : integer := USE_FILE;	-- 1 to us a sample file ; 0 to sample input
			ADC_FILE : string := ADC_FILE;	-- file containing adc samples
			NUMSAMPLES : integer := NUMSAMPLES;	-- number of samples in file
			
			VERBOSE : integer := 1
		);
	port(	
		reset, convstb, csb, rdb : in bit;
		eocb : out bit;
		in1 : in integer;
		db : out std_logic_vector( (ND - 1) downto 0)
	);
end component;

	signal reset, convstb, rdb : bit;
	signal csb : bit := '1';
	signal eocb : bit;
	signal in1 : integer;
	signal db : std_logic_vector( (ND - 1) downto 0);
	signal clk : bit;
begin
	a1 : adc
		port map( reset, convstb, csb, rdb, eocb, in1, db);
	
	reset <= '1' after 0 ns, '0' after CLKTIME;
	csb <= reset;
	
	c : process
	begin
		clk <= not clk;
		wait for CLKTIME;
	end process;
	
	main : process( reset, clk, eocb)
	begin
		if (reset = '1') then
			convstb <= '1';
			rdb <= '1';
--			csb <= '0';
			in1 <= 66;
		else
			if (eocb'event and eocb = '1') then
				convstb <= '0' after 0 ns, '1' after CLKTIME;
			elsif (eocb'event and eocb = '0') then
				rdb <= '0' after 0 ns, '1' after CLKTIME;
			end if;
		end if;
	end process;
end architecture;










