-- test bench

library ieee;
use ieee.std_logic_1164.all;		-- std_logic_vector
use ieee.std_logic_arith.all;		-- conv_std_logic
use ieee.std_logic_unsigned.all;	-- ditto

entity tb_delay_line is
end entity;

architecture beh of tb_delay_line is
	constant CLKTIME : time := 5 ns;
	constant NA : integer := 5;		-- 2**5=32 samples
	constant ND : integer := 8;		-- 8 bits / sample
	constant DOWN : integer := 0;
	
component delay_line is
	generic(	NA : integer := 5;	-- address width
			ND : integer := 8);	-- data width

	port(	delay_line_in : in std_logic_vector( (ND-1) downto 0);
		delay_line_address : in std_logic_vector( (NA-1) downto 0);
		delay_line_sample_shift, reset, clk : in bit;
		delay_line_out : out std_logic_vector( (ND-1) downto 0);

		mem0 : out std_logic_vector( (ND - 1) downto 0);
		mem1 : out std_logic_vector( (ND - 1) downto 0);
		mem2 : out std_logic_vector( (ND - 1) downto 0);
		mem3 : out std_logic_vector( (ND - 1) downto 0);
		mem4 : out std_logic_vector( (ND - 1) downto 0)
		);
end component;

	signal line_in : std_logic_vector( (ND-1) downto 0);
	signal address : std_logic_vector( (NA-1) downto 0);
	signal shift, reset, clk : bit;
	signal line_out : std_logic_vector( 7 downto 0);
	signal countd, counta : integer;
	signal mem0 : std_logic_vector( (ND - 1) downto 0);
	signal mem1 : std_logic_vector( (ND - 1) downto 0);
	signal mem2 : std_logic_vector( (ND - 1) downto 0);
	signal mem3 : std_logic_vector( (ND - 1) downto 0);
	signal mem4 : std_logic_vector( (ND - 1) downto 0);
begin
	d1 : delay_line
		port map( line_in, address, shift, reset, clk, line_out, mem0, mem1, mem2, mem3, mem4);
	
	reset <= '1' after 0 ns, '0' after CLKTIME;
	address <= conv_std_logic_vector( counta, NA);
	line_in <= conv_std_logic_vector( countd, ND);
	
	c : process
	begin
		clk <= not clk;
		wait for CLKTIME;
	end process;
	d : process( clk, reset)
	begin
		if (reset = '1') then
			countd <= 0;
--		elsif (clk'event and clk = '0') then
		elsif (clk'event) then
			countd <= countd + 1;
		end if;
	end process;
	a : process( clk, reset)
	begin
		if (DOWN = 1) then
			if (reset = '1') then
				counta <= NA - 1;
			elsif (clk'event) then
				if (counta = 0) then
					counta <= NA - 1;
					shift <= '1';
				else
					counta <= counta - 1;
					shift <= '0';
				end if;
			end if;
		else
			if (reset = '1') then
				counta <= 0;
			elsif (clk'event) then
				if (counta >= NA - 1) then
					counta <= 0;
				else
					counta <= counta + 1;
					if (counta = 0) then
						shift <= '1';
					else
						shift <= '0';
					end if;
				end if;
			end if;
		end if;
	end process;
end architecture;

-- old tb with static pulses

--architecture beh0 of tb_delay_line is
--	constant CLKTIME : time := 5 ns;
--	constant NA : integer := 5;		-- 2**5=32 samples
--	constant ND : integer := 8;		-- 8 bits / sample
--	
--component delay_line is
--	generic(	NA : integer := NA;	-- address width
--			ND : integer := ND);	-- data width
--
--	port(	delay_line_in : in std_logic_vector( (ND-1) downto 0);
--		delay_line_address : in std_logic_vector( (NA-1) downto 0);
--		delay_line_sample_shift, reset, clk : in bit;
--		delay_line_out : out std_logic_vector( (ND-1) downto 0));
--end component;
--
--	signal line_in : std_logic_vector( (ND-1) downto 0);
--	signal address : std_logic_vector( (NA-1) downto 0);
--	signal shift, reset, clk : bit;
--	signal line_out : std_logic_vector( 7 downto 0);
--begin
--	d1 : delay_line
--		port map( line_in, address, shift, reset, clk, line_out);
--	
--	reset <= '1' after 0 ns, '0' after 2 ns;
--	address <=	--conv_std_logic_vector( 0, NA) after 0 ns,
--		 	conv_std_logic_vector( 4, NA) after 56 ns,
--			conv_std_logic_vector( 3, NA) after 66 ns,
--			conv_std_logic_vector( 2, NA) after 76 ns,
--			conv_std_logic_vector( 1, NA) after 86 ns,
--			conv_std_logic_vector( 0, NA) after 96 ns,
--			(others => 'X') after 106 ns;
--			
--	line_in <=	conv_std_logic_vector( 66, ND) after 4 ns,
--			conv_std_logic_vector( 67, ND) after 14 ns,
--			conv_std_logic_vector( 68, ND) after 24 ns,
--			conv_std_logic_vector( 69, ND) after 34 ns,
--			conv_std_logic_vector( 70, ND) after 44 ns,
--			conv_std_logic_vector( 01, ND) after 54 ns;
--	shift <= '0' after 0 ns, '1' after 3 ns, '0' after 56 ns;
--	
--	c : process
--	begin
--		clk <= not clk;
--		wait for CLKTIME;
--	end process;
--end architecture;


