-- test bench

library ieee;
use ieee.std_logic_1164.all;		-- std_logic_vector
use ieee.std_logic_arith.all;		-- conv_std_logic
use ieee.std_logic_unsigned.all;	-- ditto

entity tb_fir is
end entity;

architecture beh of tb_fir is
--	constant CLKTIME : time := 25 ns;		-- 20 MHz Global Clock
	constant CLKTIME : time := 12.5 ns;		-- 40 MHz Global Clock
--	constant CLKTIME : time := 7.5757 ns;		-- 66 MHz Global Clock
	
	constant NA : integer := 5;	-- address width
	constant ND : integer := 8;	-- data width
	constant NDO : integer := 21;	-- data width
	constant NDB : integer := 21;	-- buffer width

	constant USE_FILE : integer := 1;	-- 1 to us a sample file ; 0 to sample input
	constant NUMSAMPLES : integer := 400;	-- number of samples max in file

--testbench files-----------------------------------------------------------------------------
--	constant ROM_FILE : string := "romcoefs.txt";	-- file containing rom coefs
--	constant ADC_FILE : string := "adcremi.txt";	-- file containing adc samples
--	constant NUMSAMPLES : integer := 32;	-- number of samples in file

--real world files----------------------------------------------------------------------------
--	constant ROM_FILE : string := "testing/zROM_sci.txt";	-- file containing rom coefs
	constant ROM_FILE : string := "testing/zROM_cime.txt";	-- file containing rom coefs
--	constant ROM_FILE : string := "testing/zROM_prost.txt";	-- file containing rom coefs

--	constant ADC_FILE : string := "testing/zADC_sin10k.txt";	-- file containing adc samples
--	constant ADC_FILE : string := "testing/zADC_sin20k.txt";	-- file containing adc samples
--	constant ADC_FILE : string := "testing/zADC_sin34k.txt";	-- file containing adc samples
	constant ADC_FILE : string := "testing/zADC_sin60k.txt";	-- file containing adc samples
--	constant ADC_FILE : string := "testing/zADC_sin120k.txt";	-- file containing adc samples

--	constant ADC_FILE : string := "testing/zADC_sin5k.txt";	-- file containing adc samples

component fir is
	generic(	NA : integer := NA;	-- address width
			ND : integer := ND;	-- data width
			NDO : integer := NDO;	-- data out width
			ROM_FILE : string := ROM_FILE);	-- file containing rom coefs

	port(	filter_in : in std_logic_vector( (ND - 1) downto 0);
		clk, reset, adc_busy : in bit;
		adc_convstb, adc_rd_csb : out bit;
		filter_out : out std_logic_vector( (ND - 1) downto 0)
	);
end component;

component adc is
	generic(	ND : integer := ND;	-- data width
			USE_FILE : integer := USE_FILE;	-- 1 to us a sample file ; 0 to sample input
			ADC_FILE : string := ADC_FILE;	-- file containing adc samples
			NUMSAMPLES : integer := NUMSAMPLES;	-- number of samples in file
			
			VERBOSE : integer := 0
		);
	port(	
		reset, convstb, csb, rdb : in bit;
		eocb : out bit;
		in1 : in integer;
		db : out std_logic_vector( (ND - 1) downto 0)
	);
end component;

	signal clk, reset : bit;
	signal filter_out : std_logic_vector( (NDB - 1) downto 0);

	signal adc_busy : bit;
	signal adc_input : integer := 66;
	signal adc_output : std_logic_vector( (ND - 1) downto 0);
	signal adc_csb : bit;
	signal adc_convstb, adc_rdb : bit;

begin
	f1 : fir
		port map( adc_output, clk, reset, adc_busy, adc_convstb, adc_rdb, filter_out
			);
	
	ad1 : adc
		port map( reset, adc_convstb, adc_csb, adc_rdb, adc_busy, adc_input, adc_output
			);

	reset <= '1' after 0 ns, '0' after CLKTIME;

	adc_csb <= reset;

	c : process
	begin
		clk <= not clk;
		wait for CLKTIME;
	end process;
	
end architecture;





