-- test bench

library ieee;
use ieee.std_logic_1164.all;		-- std_logic_vector
use ieee.std_logic_arith.all;		-- conv_std_logic
use ieee.std_logic_unsigned.all;	-- ditto

entity tb_fsm is
end entity;

architecture beh of tb_fsm is
	constant CLKTIME : time := 10 ns;
	constant NA : integer := 2;
	constant ADC_RESETT: integer := 10;
	constant ADC_CONVT: integer := 30;
	constant ND : integer := 8;	-- data width
	constant USE_FILE : integer := 1;	-- 1 to us a sample file ; 0 to sample input
	constant ADC_FILE : string := "adc.txt";	-- file containing adc samples
	constant NUMSAMPLES : integer := 10;	-- number of samples in file
	
component fsm is
	generic(	NA : integer := NA);
	
	port(	clk, reset, adc_busy : in bit;
		adc_convstb, adc_rd_csb : out bit;
		rom_address : out std_logic_vector( (NA-1) downto 0);
		delay_line_address : out std_logic_vector( (NA-1) downto 0);
		delay_line_sample_shift, accu_ctrl, buff_oe : out bit
	);

end component;

component adc is
	generic(	ND : integer := ND;	-- data width
			USE_FILE : integer := USE_FILE;	-- 1 to us a sample file ; 0 to sample input
			ADC_FILE : string := ADC_FILE;	-- file containing adc samples
			NUMSAMPLES : integer := NUMSAMPLES	-- number of samples in file
		);
	port(	
		reset, convstb, csb, rdb : in bit;
		eocb : out bit;
		in1 : in integer;
		db : out std_logic_vector( (ND - 1) downto 0)
	);
end component;

	signal clk, reset, adc_busy : bit;
	signal adc_convstb, adc_rd_csb : bit;
	signal rom_address : std_logic_vector( (NA - 1) downto 0);
	signal delay_line_address : std_logic_vector( (NA - 1) downto 0);
	signal delay_line_sample_shift, accu_ctrl, buff_oe : bit;

	signal csb : bit := '1';
	signal in1 : integer;
	signal db : std_logic_vector( (ND - 1) downto 0);

begin
	f1 : fsm
		port map( clk, reset, adc_busy, adc_convstb, adc_rd_csb, rom_address, 
			delay_line_address, delay_line_sample_shift, accu_ctrl, buff_oe
			);

	a1 : adc
		port map( reset, adc_convstb, csb, adc_rd_csb, adc_busy, in1, db);
	
	reset <= '1' after 0 ns, '0' after CLKTIME;
	csb <= reset;
	
	c : process
	begin
		clk <= not clk;
		wait for CLKTIME;
	end process;
	
end architecture;

-- old architecture simulating an ad7822 in a process
--
--architecture beh0 of tb_fsm is
--	constant CLKTIME : time := 5 ns;
--	constant NA : integer := 2;
--	constant ADC_RESETT: integer := 10;
--	constant ADC_CONVT: integer := 30;
--component fsm is
--	generic(	NA : integer := NA);
--	
--	port(	state0 : out integer;
--		clk, reset, adc_busy : in bit;
--		adc_convstb, adc_rd_csb : out bit;
--		rom_address : out std_logic_vector( (NA - 1) downto 0);
--		delay_line_address : out std_logic_vector( (NA - 1) downto 0);
--		delay_line_sample_shift, accu_ctrl, buff_oe : out bit);
--end component;
--
--	signal clk, reset, adc_busy : bit;
--	signal adc_convstb, adc_rd_csb : bit;
--	signal rom_address : std_logic_vector( (NA - 1) downto 0);
--	signal delay_line_address : std_logic_vector( (NA - 1) downto 0);
--	signal delay_line_sample_shift, accu_ctrl, buff_oe : bit;
--	signal state : integer;
--
--	type adc_fsm is ( adc_reset, adc_stop, adc_run);
--	signal adc_state : adc_fsm;
--	signal adc_count : integer;
--	signal adc_clk : bit;
--begin
--	f1 : fsm
--		port map( state, clk, reset, adc_busy, adc_convstb, adc_rd_csb, rom_address, delay_line_address, delay_line_sample_shift, accu_ctrl, buff_oe);
--
--	reset <= '1' after 0 ns, '0' after 3 ns;
--		
--	c : process
--	begin
--		clk <= not clk;
--		wait for CLKTIME;
--	end process;
--	
--	-- ADC : AD7822 
--	--
--	-- 8bit half-flash ADC with 420ns Conv Time
--	--
--	a_clk : process
--	begin
--		adc_clk <= not adc_clk;
--		wait for 1 ns;
--	end process;
--	adc : process( reset, adc_clk, adc_convstb)
--	begin
--		if (reset = '1') then
--			adc_busy <= '1';
--			adc_state <= adc_reset;
--			adc_count <= ADC_RESETT;
--		else
--			if (adc_clk'event or adc_convstb = '0') then
--				case adc_state is
--					when adc_reset =>
--						if (adc_count = 0) then
--							adc_state <= adc_stop;
--							adc_busy <= '0';
--						else
--							adc_count <= adc_count - 1;
--						end if;
--					when adc_stop =>
--						if (adc_convstb = '0') then
--							adc_busy <= '1';
--							adc_count <= ADC_CONVT;
--							adc_state <= adc_run;
--						end if;
--					when adc_run =>
--						if (adc_count = 0) then
--							adc_busy <= '0';
--							adc_state <= adc_stop;
--						else
--							adc_count <= adc_count - 1;
--						end if;
--				end case;
--			end if;
--		end if;
--	end process;
--end architecture;





















