-- test bench

library ieee;
use ieee.std_logic_1164.all;		-- std_logic_vector
use ieee.std_logic_arith.all;		-- conv_std_logic
use ieee.std_logic_unsigned.all;	-- ditto

entity tb_rom is
end entity;

architecture beh of tb_rom is
	constant NA : integer := 5;	-- address width
	constant ND : integer := 8;	-- data width
	constant CLKTIME : time := 10 ns;
	constant ROM_FILE : string := "romcoefs.txt";	-- file containing rom coefs

component rom is
	generic(	NA : integer := NA;	-- address width
			ND : integer := ND;	-- data width
			ROM_FILE : string := ROM_FILE);	-- file containing rom coefs

	port(	rom_address : in std_logic_vector( (NA-1) downto 0);
		reset, clk : in bit;
		rom_out : out std_logic_vector( (ND-1) downto 0)
		);

end component;

	signal rom_address : std_logic_vector( (NA-1) downto 0);
	signal reset, clk : bit;
	signal rom_out : std_logic_vector( (ND-1) downto 0);

	signal count : integer;

begin
	r1 : rom
		port map( rom_address, reset, clk, rom_out);
	
	reset <= '1' after 0 ns, '0' after CLKTIME;
	
	rom_address <= conv_std_logic_vector( count, NA);
	
	c : process
	begin
		clk <= not clk;
		wait for CLKTIME;
	end process;
	
	p : process( reset, clk)
	begin
		if (reset = '1') then
			count <= 0;
		elsif (clk'event and clk = '1') then
			count <= count + 1;
		end if;
	end process;
end architecture;


